#define MOZ_SOURCE_STAMP ae5275b8c53ba76cb98576e4e2a3031b0d659ba3
#define MOZ_SOURCE_REPO https://hg.mozilla.org/releases/mozilla-beta
#define MOZ_SOURCE_URL https://hg.mozilla.org/releases/mozilla-beta/rev/ae5275b8c53ba76cb98576e4e2a3031b0d659ba3
