/* THIS FILE IS AUTOGENERATED FROM MediaTrackSettings.webidl BY Codegen.py - DO NOT EDIT */

#ifndef mozilla_dom_MediaTrackSettingsBinding_h
#define mozilla_dom_MediaTrackSettingsBinding_h

#include "js/RootingAPI.h"
#include "jspubtd.h"
#include "mozilla/ErrorResult.h"
#include "mozilla/dom/BindingDeclarations.h"
#include "mozilla/dom/Nullable.h"

namespace mozilla {
namespace dom {

struct MediaTrackSettingsAtoms;
struct NativePropertyHooks;
class ProtoAndIfaceCache;

} // namespace dom
} // namespace mozilla

namespace mozilla {
namespace dom {

struct MediaTrackSettings : public DictionaryBase
{
  Optional<int64_t> mBrowserWindow;
  Optional<nsString> mDeviceId;
  Optional<bool> mEchoCancellation;
  Optional<nsString> mFacingMode;
  Optional<double> mFrameRate;
  Optional<int32_t> mHeight;
  Optional<nsString> mMediaSource;
  Optional<bool> mMozAutoGainControl;
  Optional<bool> mMozNoiseSuppression;
  Optional<bool> mScrollWithPage;
  Optional<int32_t> mViewportHeight;
  Optional<int32_t> mViewportOffsetX;
  Optional<int32_t> mViewportOffsetY;
  Optional<int32_t> mViewportWidth;
  Optional<int32_t> mWidth;

  MediaTrackSettings();

  explicit inline MediaTrackSettings(const FastDictionaryInitializer& )
  {
    // Do nothing here; this is used by our "Fast" subclass
  }

  explicit inline MediaTrackSettings(const MediaTrackSettings& aOther)
  {
    *this = aOther;
  }

  bool
  Init(JSContext* cx, JS::Handle<JS::Value> val, const char* sourceDescription = "Value", bool passedToJSImpl = false);

  bool
  Init(const nsAString& aJSON);

  bool
  ToObjectInternal(JSContext* cx, JS::MutableHandle<JS::Value> rval) const;

  bool
  ToJSON(nsAString& aJSON) const;

  void
  TraceDictionary(JSTracer* trc);

  void
  operator=(const MediaTrackSettings& aOther);

private:
  static bool
  InitIds(JSContext* cx, MediaTrackSettingsAtoms* atomsCache);
};

namespace binding_detail {
struct FastMediaTrackSettings : public MediaTrackSettings
{
  inline FastMediaTrackSettings()
    : MediaTrackSettings(FastDictionaryInitializer())
  {
    // Doesn't matter what int we pass to the parent constructor
  }
};
} // namespace binding_detail


} // namespace dom
} // namespace mozilla

#endif // mozilla_dom_MediaTrackSettingsBinding_h
